/*!---------------------------------------------------------
* Copyright (C) Microsoft Corporation. All rights reserved.
*--------------------------------------------------------*/
/// <reference path="../declare/node.d.ts" />
/// <reference path="../declare/atom-renderer.d.ts" />
'use strict';
var ipcbus = require('../ipcbus');
function generateId() {
    return Math.floor(Math.random() * Math.pow(10, 10)).toString(16);
}
function fatal(err) {
    console.error(err);
    process.exit(1);
}
function main(argv) {
    if (argv.length !== 5) {
        return fatal('Wrong number of arguments');
    }
    if (!process.env['VSCODE_IPC_HOOK']) {
        return fatal('Missing ipc hook');
    }
    if (!process.env['VSCODE_GIT_REQUEST_ID']) {
        return fatal('Missing git id');
    }
    var id = process.env['VSCODE_GIT_REQUEST_ID'];
    var request = argv[2];
    var host = argv[4].substring(1, argv[4].length - 2);
    try {
        ipcbus.connect(process.env['VSCODE_IPC_HOOK'], function (err, bus) {
            if (err) {
                return fatal(err);
            }
            bus.emit('git:askpass', {
                id: id,
                host: host,
                command: process.env['MONACO_GIT_COMMAND']
            });
            bus.on('git:askpass:' + id, function (result) {
                if (result) {
                    console.log(/^Username$/i.test(request) ? result.username : result.password);
                }
                bus.disconnect();
                setTimeout(function () { return process.exit(0); }, 0);
            });
        });
    }
    catch (err) {
        fatal(err);
    }
}
if (require.main === module) {
    process.on('uncaughtException', fatal);
    main(process.argv);
}
