/*!---------------------------------------------------------
* Copyright (C) Microsoft Corporation. All rights reserved.
*--------------------------------------------------------*/
/// <reference path="./declare/node.d.ts" />
/// <reference path="./declare/atom-browser.d.ts" />
'use strict';
var path = require('path');
var objects = require('./objects');
var sepCode = path.sep.charCodeAt(0);
function isEqualOrParent(pathToCheck, candidate) {
    if (!pathToCheck || !candidate) {
        return false;
    }
    pathToCheck = path.normalize(pathToCheck).toLowerCase();
    candidate = path.normalize(candidate).toLowerCase();
    if (pathToCheck === candidate) {
        return true;
    }
    if (pathToCheck.indexOf(candidate) !== 0) {
        return false;
    }
    var idx = candidate.length;
    if (candidate.charCodeAt(idx - 1) === sepCode) {
        idx -= 1;
    }
    return pathToCheck.charCodeAt(idx) === path.sep.charCodeAt(0);
}
exports.isEqualOrParent = isEqualOrParent;
function parseLineAndColumnAware(rawPath) {
    var segments = rawPath.split(':'); // C:\file.txt:<line>:<column>
    var path;
    var line = null;
    var column = null;
    segments.forEach(function (segment) {
        var segmentAsNumber = Number(segment);
        if (!objects.isNumber(segmentAsNumber)) {
            path = !!path ? [path, segment].join(':') : segment; // a colon can well be part of a path (e.g. C:\...)
        }
        else if (line === null) {
            line = segmentAsNumber;
        }
        else if (column === null) {
            column = segmentAsNumber;
        }
    });
    return {
        path: path,
        line: line !== null ? line : void 0,
        column: column !== null ? column : line !== null ? 1 : void 0 // if we have a line, make sure column is also set
    };
}
exports.parseLineAndColumnAware = parseLineAndColumnAware;
function toLineAndColumnPath(parsedPath) {
    var segments = [parsedPath.path];
    if (objects.isNumber(parsedPath.line)) {
        segments.push(String(parsedPath.line));
    }
    if (objects.isNumber(parsedPath.column)) {
        segments.push(String(parsedPath.column));
    }
    return segments.join(':');
}
exports.toLineAndColumnPath = toLineAndColumnPath;
